@@ -123,11 +123,8 @@ public class SCRDescriptorGenerator {
         }
 
         this.logger.debug("..using output directory: " + this.options.getOutputDirectory());
-        this.logger.debug("..using scr name: " + this.options.getSCRName());
-        this.logger.debug("..using metatype name: " + this.options.getMetaTypeName());
         this.logger.debug("..strict mode: " + this.options.isStrictMode());
         this.logger.debug("..generating accessors: " + this.options.isGenerateAccessors());
-        this.logger.debug("..generating separate descs: " + this.options.isGenerateSeparateDescriptors());
 
         // check speck version configuration
         SpecVersion specVersion = options.getSpecVersion();
@@ -227,7 +224,7 @@ public class SCRDescriptorGenerator {
         }
 
         // and generate files
-        result.setMetatypeFiles(MetaTypeIO.generateDescriptors(module, this.options, this.logger));
+        result.setMetatypeFiles(MetaTypeIO.generateDescriptors(module, this.project, this.options, this.logger));
         result.setScrFiles(ComponentDescriptorIO.generateDescriptorFiles(module, this.options, logger));
 
         return result;
@@ -235,6 +232,11 @@ public class SCRDescriptorGenerator {
 
     private void generateMethods(final ComponentContainer container) throws SCRDescriptorException {
         for (final ReferenceDescription ref : container.getReferences().values()) {
+            // skip refs without a interface name (validate will be called next)
+            if (StringUtils.isEmpty(ref.getInterfaceName())) {
+                continue;
+            }
+
             // if this is a field with a single cardinality,
             // we look for the bind/unbind methods
             // and create them if they are not availabe
@@ -278,15 +280,20 @@ public class SCRDescriptorGenerator {
                                     this.project.getClassLoader(),
                                     this.project.getClassesDirectory(),
                                     this.logger);
+                    // set a flag for validation
+                    ref.setBindMethodCreated(createBind);
+                    ref.setUnbindMethodCreated(createUnbind);
                 }
             }
         }
     }
+
     /**
      * Create the SCR objects based on the descriptions
      */
     private ComponentContainer createComponent(final ClassDescription desc,
-                    final IssueLog iLog) {
+                    final IssueLog iLog)
+    throws SCRDescriptorException {
         final ComponentDescription componentDesc = desc.getDescription(ComponentDescription.class);
 
         final SpecVersion intitialComponentSpecVersion = componentDesc.getSpecVersion();
@@ -307,13 +314,9 @@ public class SCRDescriptorGenerator {
             ocd.setId( componentDesc.getName() );
             if ( componentDesc.getLabel() != null ) {
                 ocd.setName( componentDesc.getLabel() );
-            } else {
-                ocd.setName( "%" + componentDesc.getName() + ".name");
             }
             if ( componentDesc.getDescription() != null ) {
                 ocd.setDescription( componentDesc.getDescription() );
-            } else {
-                ocd.setDescription( "%" + componentDesc.getName() + ".description");
             }
 
             // Factory pid
@@ -328,6 +331,17 @@ public class SCRDescriptorGenerator {
         } else {
             ocd = null;
         }
+        // metatype checks if metatype is not generated (FELIX-4033)
+        if ( !componentDesc.isAbstract() && !componentDesc.isCreateMetatype() ) {
+            if ( componentDesc.getLabel() != null && componentDesc.getLabel().trim().length() > 0 ) {
+                iLog.addWarning(" Component " + componentDesc.getName() + " has set a label. However metatype is set to false. This label is ignored.",
+                        desc.getSource());
+            }
+            if ( componentDesc.getDescription() != null && componentDesc.getDescription().trim().length() > 0 ) {
+                iLog.addWarning(" Component " + componentDesc.getName() + " has set a description. However metatype is set to false. This description is ignored.",
+                        desc.getSource());
+            }
+        }
 
         ClassDescription current = desc;
         boolean inherit;
@@ -336,6 +350,12 @@ public class SCRDescriptorGenerator {
             inherit = (cd == null ? true : cd.isInherit());
 
             if ( cd != null ) {
+                if ( current != desc ) {
+                    iLog.addWarning(" Component " + componentDesc.getName() + " is using the " +
+                                    "deprecated inheritance feature and inherits from " + current.getDescribedClass().getName() +
+                                    ". This feature will be removed in future versions.",
+                                    desc.getSource());
+                }
                 // handle enabled and immediate
                 if ( componentDesc.getEnabled() == null ) {
                     componentDesc.setEnabled(cd.getEnabled());
@@ -362,13 +382,14 @@ public class SCRDescriptorGenerator {
                     // policy requires 1.1
                     componentDesc.setSpecVersion(SpecVersion.VERSION_1_1);
                 }
-            }
 
+            }
             // services, properties, references
             this.processServices(current, container);
             this.processProperties(current, container, ocd);
             this.processReferences(current, container);
 
+
             // go up in the class hierarchy
             if ( !inherit || current.getDescribedClass().getSuperclass() == null ) {
                 current = null;
@@ -410,14 +431,18 @@ public class SCRDescriptorGenerator {
         // global properties
         this.processGlobalProperties(desc, container.getProperties());
 
-        // PID handling
-        if ( componentDesc.isCreatePid() && !container.getProperties().containsKey(org.osgi.framework.Constants.SERVICE_PID)) {
-            final PropertyDescription pid = new PropertyDescription(null);
-            pid.setName( org.osgi.framework.Constants.SERVICE_PID );
-            pid.setValue( componentDesc.getName() );
-            pid.setType(PropertyType.String);
-
-            container.getProperties().put(org.osgi.framework.Constants.SERVICE_PID, pid);
+        // check lifecycle methods
+        if ( componentDesc.getActivate() == null ) {
+            final Validator.MethodResult result = Validator.findLifecycleMethod(project, container, "activate", true);
+            if ( result.method != null ) {
+                componentDesc.setSpecVersion(result.requiredSpecVersion);
+            }
+        }
+        if ( componentDesc.getDeactivate() == null ) {
+            final Validator.MethodResult result = Validator.findLifecycleMethod(project, container, "deactivate", false);
+            if ( result.method != null ) {
+                componentDesc.setSpecVersion(result.requiredSpecVersion);
+            }
         }
 
         // check if component has spec version configured but requires a higher one
@@ -451,6 +476,21 @@ public class SCRDescriptorGenerator {
         }
     }
 
+    private boolean isPrivateProperty(final String name) {
+        final boolean isPrivate;
+        if (org.osgi.framework.Constants.SERVICE_RANKING.equals(name)
+                || org.osgi.framework.Constants.SERVICE_PID.equals(name)
+                || org.osgi.framework.Constants.SERVICE_DESCRIPTION.equals(name)
+                || org.osgi.framework.Constants.SERVICE_VENDOR.equals(name)
+                || ConfigurationAdmin.SERVICE_BUNDLELOCATION.equals(name)
+                || ConfigurationAdmin.SERVICE_FACTORYPID.equals(name) ) {
+                isPrivate = true;
+            } else {
+                isPrivate = false;
+            }
+        return isPrivate;
+    }
+
     /**
      * Process property directives
      */
@@ -460,70 +500,75 @@ public class SCRDescriptorGenerator {
                     final MetatypeContainer ocd) {
         for(final PropertyDescription pd : current.getDescriptions(PropertyDescription.class)) {
 
-            if ( this.testProperty(current, component.getProperties(), pd, current == component.getClassDescription()) && ocd != null) {
+            if ( this.testProperty(current, component.getProperties(), pd, current == component.getClassDescription()) ) {
+                final String name = pd.getName();
+                if ( org.osgi.framework.Constants.SERVICE_ID.equals(name) ) {
+                    iLog.addError("Class " + current.getDescribedClass().getName() + " is declaring " +
+                                  "the protected property 'service.id'.", current.getSource() );
+                    continue;
 
-                // metatype - is this property private?
-                final boolean isPrivate;
-                if ( pd.isPrivate() != null ) {
-                    isPrivate = pd.isPrivate();
-                } else {
-                    final String name = pd.getName();
-                    if (org.osgi.framework.Constants.SERVICE_RANKING.equals(name)
-                        || org.osgi.framework.Constants.SERVICE_PID.equals(name)
-                        || org.osgi.framework.Constants.SERVICE_DESCRIPTION.equals(name)
-                        || org.osgi.framework.Constants.SERVICE_ID.equals(name)
-                        || org.osgi.framework.Constants.SERVICE_VENDOR.equals(name)
-                        || ConfigurationAdmin.SERVICE_BUNDLELOCATION.equals(name)
-                        || ConfigurationAdmin.SERVICE_FACTORYPID.equals(name) ) {
-                        isPrivate = true;
-                    } else {
-                        isPrivate = false;
-                    }
                 }
-                if ( !isPrivate ) {
-                    final MetatypeAttributeDefinition ad = new MetatypeAttributeDefinition();
-                    ocd.getProperties().add(ad);
-                    ad.setId(pd.getName());
-                    ad.setType(pd.getType().name());
+                if ( ocd != null) {
 
-                    if (pd.getLabel() != null ) {
-                        ad.setName(pd.getLabel());
+                    // metatype - is this property private?
+                    final boolean isPrivate;
+                    if ( pd.isPrivate() != null ) {
+                        isPrivate = pd.isPrivate();
                     } else {
-                        ad.setName("%" + pd.getName() + ".name");
-                    }
-                    if (pd.getDescription() != null ) {
-                        ad.setDescription(pd.getDescription());
-                    } else {
-                        ad.setDescription("%" + pd.getName() + ".description");
-                    }
-
-                    if ( pd.getUnbounded() == PropertyUnbounded.DEFAULT ) {
-                        if ( pd.getCardinality() != 0 ) {
-                            ad.setCardinality(pd.getCardinality());
+                        if (isPrivateProperty(name) ) {
+                            isPrivate = true;
+                        } else {
+                            isPrivate = false;
                         }
-                    } else if ( pd.getUnbounded() == PropertyUnbounded.ARRAY ) {
-                        // unlimited array
-                        ad.setCardinality(new Integer(Integer.MAX_VALUE));
-                    } else {
-                        // unlimited vector
-                        ad.setCardinality(new Integer(Integer.MIN_VALUE));
                     }
+                    if ( !isPrivate ) {
+                        final MetatypeAttributeDefinition ad = new MetatypeAttributeDefinition();
+                        ocd.getProperties().add(ad);
+                        ad.setId(pd.getName());
+                        ad.setType(pd.getType().name());
 
-                    ad.setDefaultValue(pd.getValue());
-                    ad.setDefaultMultiValue(pd.getMultiValue());
+                        if (pd.getLabel() != null ) {
+                            ad.setName(pd.getLabel());
+                        }
+                        if (pd.getDescription() != null ) {
+                            ad.setDescription(pd.getDescription());
+                        }
 
-                    // check options
-                    final String[] parameters = pd.getOptions();
-                    if ( parameters != null && parameters.length > 0 ) {
-                        final Map<String, String> options = new LinkedHashMap<String, String>();
-                        for (int j=0; j < parameters.length; j=j+2) {
-                            final String optionLabel = parameters[j];
-                            final String optionValue = (j < parameters.length-1) ? parameters[j+1] : null;
-                            if (optionValue != null) {
-                                options.put(optionLabel, optionValue);
+                        if ( pd.getUnbounded() == PropertyUnbounded.DEFAULT ) {
+                            if ( pd.getCardinality() != 0 ) {
+                                ad.setCardinality(pd.getCardinality());
                             }
+                        } else if ( pd.getUnbounded() == PropertyUnbounded.ARRAY ) {
+                            // unlimited array
+                            ad.setCardinality(new Integer(Integer.MAX_VALUE));
+                        } else {
+                            // unlimited vector
+                            ad.setCardinality(new Integer(Integer.MIN_VALUE));
                         }
-                        ad.setOptions(options);
+
+                        ad.setDefaultValue(pd.getValue());
+                        ad.setDefaultMultiValue(pd.getMultiValue());
+
+                        // check options
+                        final String[] parameters = pd.getOptions();
+                        if ( parameters != null && parameters.length > 0 ) {
+                            final Map<String, String> options = new LinkedHashMap<String, String>();
+                            for (int j=0; j < parameters.length; j=j+2) {
+                                final String optionLabel = parameters[j];
+                                final String optionValue = (j < parameters.length-1) ? parameters[j+1] : null;
+                                if (optionValue != null) {
+                                    options.put(optionLabel, optionValue);
+                                }
+                            }
+                            ad.setOptions(options);
+                        }
+                    }
+                } else {
+                    // additional metatype checks (FELIX-4033)
+                    if ( pd.isPrivate() != null && pd.isPrivate() ) {
+                        iLog.addWarning("Property " + pd.getName() + " in class "
+                                + current.getDescribedClass().getName() + " is set as private. " +
+                                "This is redundant as no metatype will be generated.", current.getSource() );
                     }
                 }
             }
@@ -602,24 +647,26 @@ public class SCRDescriptorGenerator {
 
             this.testReference(current, component.getReferences(), rd, component.getClassDescription() == current);
 
-            // check for method signature
-            try {
-                final Validator.MethodResult bindMethod = Validator.findMethod(this.project, this.options, current, rd,
-                        rd.getBind() == null ? "bind" : rd.getBind());
-                if ( bindMethod != null ) {
-                    component.getComponentDescription().setSpecVersion(bindMethod.requiredSpecVersion);
-                }
+            // check for method signature - if interface name is set (empty interface name will fail during validate)
+            if (!StringUtils.isEmpty(rd.getInterfaceName())) {
 
-                final Validator.MethodResult unbindMethod = Validator.findMethod(this.project, this.options, current, rd,
-                        rd.getUnbind() == null ? "unbind" : rd.getUnbind());
-                if ( unbindMethod != null ) {
-                    component.getComponentDescription().setSpecVersion(unbindMethod.requiredSpecVersion);
-                }
+                try {
+                    final Validator.MethodResult bindMethod = Validator.findMethod(this.project, this.options, current, rd,
+                            rd.getBind() == null ? "bind" : rd.getBind());
+                    if ( bindMethod != null ) {
+                        component.getComponentDescription().setSpecVersion(bindMethod.requiredSpecVersion);
+                    }
 
-            } catch (final SCRDescriptorException sde) {
-                // this happens only if a class not found exception occurs, so we can ignore this at this point!
+                    final Validator.MethodResult unbindMethod = Validator.findMethod(this.project, this.options, current, rd,
+                            rd.getUnbind() == null ? "unbind" : rd.getUnbind());
+                    if ( unbindMethod != null ) {
+                        component.getComponentDescription().setSpecVersion(unbindMethod.requiredSpecVersion);
+                    }
+
+                } catch (final SCRDescriptorException sde) {
+                    // this happens only if a class not found exception occurs, so we can ignore this at this point!
+                }
             }
-
         }
     }
 
