@@ -18,11 +18,9 @@
  */
 package org.apache.felix.scrplugin.xml;
 
-import java.awt.Component;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
-import java.security.Provider.Service;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.StringTokenizer;
@@ -34,6 +32,7 @@ import org.apache.felix.scrplugin.Options;
 import org.apache.felix.scrplugin.SCRDescriptorException;
 import org.apache.felix.scrplugin.SCRDescriptorFailureException;
 import org.apache.felix.scrplugin.SpecVersion;
+import org.apache.felix.scrplugin.description.AbstractDescription;
 import org.apache.felix.scrplugin.description.ClassDescription;
 import org.apache.felix.scrplugin.description.ComponentConfigurationPolicy;
 import org.apache.felix.scrplugin.description.ComponentDescription;
@@ -51,7 +50,6 @@ import org.apache.felix.scrplugin.helper.ComponentContainerUtil;
 import org.apache.felix.scrplugin.helper.ComponentContainerUtil.ComponentContainerContainer;
 import org.apache.felix.scrplugin.helper.DescriptionContainer;
 import org.apache.felix.scrplugin.helper.IssueLog;
-import org.apache.felix.scrplugin.helper.StringUtils;
 import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.SAXException;
@@ -182,7 +180,7 @@ public class ComponentDescriptorIO {
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generateXML(final DescriptionContainer module,
+    private static void generateXML(final DescriptionContainer module,
             final List<ComponentContainer> components,
             final File descriptorFile,
             final Log logger) throws SAXException, IOException, TransformerException {
@@ -195,33 +193,40 @@ public class ComponentDescriptorIO {
         contentHandler.startDocument();
         contentHandler.startPrefixMapping(PREFIX, namespace);
 
-        // wrapper element to generate well formed xml
-        contentHandler.startElement("", ComponentDescriptorIO.COMPONENTS, ComponentDescriptorIO.COMPONENTS, new AttributesImpl());
         IOUtils.newline(contentHandler);
-
+        // wrapper element to generate well formed xml if 0 or more than 1 component
+        int startIndent = 0;
+        if ( components.size() != 1 ) {
+            contentHandler.startElement("", ComponentDescriptorIO.COMPONENTS, ComponentDescriptorIO.COMPONENTS, new AttributesImpl());
+            IOUtils.newline(contentHandler);
+            startIndent = 1;
+        }
         for (final ComponentContainer component : components) {
-            generateXML(namespace, module, component, contentHandler);
+            generateXML(namespace, module, component, contentHandler, startIndent);
         }
 
         // end wrapper element
-        contentHandler.endElement("", ComponentDescriptorIO.COMPONENTS, ComponentDescriptorIO.COMPONENTS);
-        IOUtils.newline(contentHandler);
+        if ( components.size() != 1 ) {
+            contentHandler.endElement("", ComponentDescriptorIO.COMPONENTS, ComponentDescriptorIO.COMPONENTS);
+            IOUtils.newline(contentHandler);
+        }
         contentHandler.endPrefixMapping(PREFIX);
         contentHandler.endDocument();
     }
 
     /**
-     * Write the xml for a {@link Component}.
+     * Write the xml for a Component
      *
      * @param component
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generateXML(final String namespace,
+    private static void generateXML(final String namespace,
             final DescriptionContainer module,
             final ComponentContainer container,
-            final ContentHandler contentHandler)
-                    throws SAXException {
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final ComponentDescription component = container.getComponentDescription();
 
         final AttributesImpl ai = new AttributesImpl();
@@ -246,37 +251,39 @@ public class ComponentDescriptorIO {
                 IOUtils.addAttribute(ai, COMPONENT_ATTR_CONFIGURATION_PID, component.getConfigurationPid());
             }
         }
-        IOUtils.indent(contentHandler, 1);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(namespace, ComponentDescriptorIO.COMPONENT, ComponentDescriptorIO.COMPONENT_QNAME, ai);
         IOUtils.newline(contentHandler);
-        generateImplementationXML(container, contentHandler);
-        if (container.getServiceDescription() != null) {
-            generateServiceXML(container.getServiceDescription(), contentHandler);
-        }
         for (final PropertyDescription property : container.getProperties().values()) {
-            generatePropertyXML(property, contentHandler);
+            generatePropertyXML(property, contentHandler, indent+1);
+        }
+        if (container.getServiceDescription() != null) {
+            generateServiceXML(container.getServiceDescription(), contentHandler, indent+1);
         }
-
         for (final ReferenceDescription reference : container.getReferences().values()) {
-            generateReferenceXML(component, module, reference, contentHandler);
+            generateReferenceXML(component, module, reference, contentHandler, indent+1);
         }
+        generateImplementationXML(container, contentHandler, indent+1);
 
-        IOUtils.indent(contentHandler, 1);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.endElement(namespace, ComponentDescriptorIO.COMPONENT, ComponentDescriptorIO.COMPONENT_QNAME);
         IOUtils.newline(contentHandler);
     }
 
     /**
-     * Write the xml for a {@link Implementation}.
+     * Write the xml for an Implementation.
      *
      * @param implementation
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generateImplementationXML(ComponentContainer component, ContentHandler contentHandler) throws SAXException {
+    private static void generateImplementationXML(final ComponentContainer component,
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final AttributesImpl ai = new AttributesImpl();
         IOUtils.addAttribute(ai, IMPLEMENTATION_ATTR_CLASS, component.getClassDescription().getDescribedClass().getName());
-        IOUtils.indent(contentHandler, 2);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.IMPLEMENTATION,
                 ComponentDescriptorIO.IMPLEMENTATION_QNAME, ai);
         contentHandler.endElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.IMPLEMENTATION,
@@ -285,43 +292,46 @@ public class ComponentDescriptorIO {
     }
 
     /**
-     * Write the xml for a {@link Service}.
+     * Write the xml for a service.
      *
      * @param service
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generateServiceXML(
+    private static void generateServiceXML(
             final ServiceDescription service,
-            final ContentHandler contentHandler)
-                    throws SAXException {
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final AttributesImpl ai = new AttributesImpl();
         IOUtils.addAttribute(ai, SERVICE_ATTR_FACTORY, String.valueOf(service.isServiceFactory()));
-        IOUtils.indent(contentHandler, 2);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.SERVICE, ComponentDescriptorIO.SERVICE_QNAME, ai);
         if (service.getInterfaces() != null && service.getInterfaces().size() > 0) {
             IOUtils.newline(contentHandler);
             for (final String interf : service.getInterfaces()) {
-                generateServiceXML(interf, contentHandler);
+                generateServiceXML(interf, contentHandler, indent+1);
             }
-            IOUtils.indent(contentHandler, 2);
+            IOUtils.indent(contentHandler, indent);
         }
         contentHandler.endElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.SERVICE, ComponentDescriptorIO.SERVICE_QNAME);
         IOUtils.newline(contentHandler);
     }
 
     /**
-     * Write the xml for a {@link Interface}.
+     * Write the xml for a interface
      *
      * @param interf
      * @param contentHandler
      * @throws SAXException
      */
-    private static void generateServiceXML(final String interfaceName, final ContentHandler contentHandler)
-            throws SAXException {
+    private static void generateServiceXML(final String interfaceName,
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final AttributesImpl ai = new AttributesImpl();
         IOUtils.addAttribute(ai, INTERFACE_ATTR_NAME, interfaceName);
-        IOUtils.indent(contentHandler, 3);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.INTERFACE, ComponentDescriptorIO.INTERFACE_QNAME,
                 ai);
         contentHandler.endElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.INTERFACE, ComponentDescriptorIO.INTERFACE_QNAME);
@@ -329,16 +339,19 @@ public class ComponentDescriptorIO {
     }
 
     /**
-     * Write the xml for a {@link Property}.
+     * Write the xml for a property.
      *
      * @param property
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generatePropertyXML(PropertyDescription property, ContentHandler contentHandler) throws SAXException {
+    private static void generatePropertyXML(final PropertyDescription property,
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final AttributesImpl ai = new AttributesImpl();
         IOUtils.addAttribute(ai, ATTR_NAME, property.getName());
-        if ( property.getType() != PropertyType.String && property.getType() != PropertyType.Passwort) {
+        if ( property.getType() != PropertyType.String && property.getType() != PropertyType.Password) {
             IOUtils.addAttribute(ai, PROPERTY_ATTR_TYPE, property.getType());
         }
         String value = property.getValue();
@@ -349,13 +362,13 @@ public class ComponentDescriptorIO {
             IOUtils.addAttribute(ai, PROPERTY_ATTR_VALUE, value);
         }
 
-        IOUtils.indent(contentHandler, 2);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.PROPERTY, ComponentDescriptorIO.PROPERTY_QNAME, ai);
         if (property.getMultiValue() != null && property.getMultiValue().length > 0) {
             // generate a new line first
             IOUtils.text(contentHandler, "\n");
             for (int i = 0; i < property.getMultiValue().length; i++) {
-                IOUtils.indent(contentHandler, 3);
+                IOUtils.indent(contentHandler, indent + 1);
                 value = property.getMultiValue()[i];
                 if ( property.getType() == PropertyType.Character || property.getType() == PropertyType.Char ) {
                     value = String.valueOf((int)value.charAt(0));
@@ -363,24 +376,25 @@ public class ComponentDescriptorIO {
                 IOUtils.text(contentHandler, value);
                 IOUtils.newline(contentHandler);
             }
-            IOUtils.indent(contentHandler, 2);
+            IOUtils.indent(contentHandler, indent);
         }
         contentHandler.endElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.PROPERTY, ComponentDescriptorIO.PROPERTY_QNAME);
         IOUtils.newline(contentHandler);
     }
 
     /**
-     * Write the xml for a {@link Reference}.
+     * Write the xml for a Reference.
      *
      * @param reference
      * @param contentHandler
      * @throws SAXException
      */
-    protected static void generateReferenceXML(final ComponentDescription component,
+    private static void generateReferenceXML(final ComponentDescription component,
             final DescriptionContainer module,
             final ReferenceDescription reference,
-            final ContentHandler contentHandler)
-                    throws SAXException {
+            final ContentHandler contentHandler,
+            final int indent)
+    throws SAXException {
         final AttributesImpl ai = new AttributesImpl();
         IOUtils.addAttribute(ai, ATTR_NAME, reference.getName());
         IOUtils.addAttribute(ai, INTERFACE_ATTR_NAME, reference.getInterfaceName());
@@ -402,7 +416,7 @@ public class ComponentDescriptorIO {
             }
         }
 
-        IOUtils.indent(contentHandler, 2);
+        IOUtils.indent(contentHandler, indent);
         contentHandler.startElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.REFERENCE, ComponentDescriptorIO.REFERENCE_QNAME,
                 ai);
         contentHandler.endElement(INNER_NAMESPACE_URI, ComponentDescriptorIO.REFERENCE, ComponentDescriptorIO.REFERENCE_QNAME);
@@ -448,6 +462,9 @@ public class ComponentDescriptorIO {
         /** XML file location. */
         private final String location;
 
+        /** Properties. */
+        private final List<AbstractDescription> propsAndRefs = new ArrayList<AbstractDescription>();
+
         /** Classloader. */
         private final ClassLoader classLoader;
 
@@ -573,7 +590,7 @@ public class ComponentDescriptorIO {
                             } else {
                                 prop.setValue(attributes.getValue(PROPERTY_ATTR_VALUE));
                             }
-                            this.currentClass.add(prop);
+                            propsAndRefs.add(prop);
                         } else {
                             // hold the property pending as we have a multi value
                             this.pendingProperty = prop;
@@ -606,7 +623,6 @@ public class ComponentDescriptorIO {
                 } else if (localName.equals(SERVICE)) {
 
                     this.currentService = new ServiceDescription(null);
-                    this.currentClass.add(this.currentService);
 
                     if (attributes.getValue(SERVICE_ATTR_FACTORY) != null) {
                         this.currentService.setServiceFactory(Boolean.valueOf(attributes.getValue(SERVICE_ATTR_FACTORY)));
@@ -665,7 +681,7 @@ public class ComponentDescriptorIO {
                         }
                     }
 
-                    this.currentClass.add(ref);
+                    propsAndRefs.add(ref);
                 }
             }
         }
@@ -685,8 +701,18 @@ public class ComponentDescriptorIO {
 
             if (SpecVersion.fromNamespaceUrl(uri) != null) {
                 if (localName.equals(COMPONENT)) {
+                    if ( this.currentClass != null && this.currentService != null ) {
+                        this.currentClass.add(currentService);
+                    }
+                    if ( this.currentClass != null ) {
+                        for(final AbstractDescription d : propsAndRefs) {
+                            this.currentClass.add(d);
+                        }
+                    }
+                    this.propsAndRefs.clear();
                     this.currentClass = null;
                     this.currentComponent = null;
+                    this.currentService = null;
                     this.isComponent = false;
                 } else if (localName.equals(PROPERTY) && this.pendingProperty != null) {
                     // now split the value
@@ -706,7 +732,7 @@ public class ComponentDescriptorIO {
                         }
                         this.pendingProperty.setMultiValue(values);
                     }
-                    this.currentClass.add(this.pendingProperty);
+                    propsAndRefs.add(this.pendingProperty);
                     this.pendingProperty = null;
                 }
             }
@@ -748,51 +774,47 @@ public class ComponentDescriptorIO {
 
         // check descriptor file
         final File descriptorDir = options.getComponentDescriptorDirectory();
-        final File descriptorFile = StringUtils.isEmpty(options.getSCRName()) ? null : new File(descriptorDir, options.getSCRName());
 
         // terminate if there is nothing else to write
         if (components.isEmpty()) {
             logger.debug("No Service Component Descriptors found in project.");
-            // remove file if it exists
-            if (descriptorFile != null && descriptorFile.exists()) {
-                logger.debug("Removing obsolete service descriptor " + descriptorFile);
-                descriptorFile.delete();
+            // remove files if it exists
+            if ( descriptorDir.exists() && !options.isIncremental()) {
+                for(final File f : descriptorDir.listFiles()) {
+                    if ( f.isFile() ) {
+                        logger.debug("Removing obsolete service descriptor " + f);
+                        f.delete();
+                    }
+                }
             }
+
             return null;
         }
-        if ( !options.isGenerateSeparateDescriptors() && descriptorFile == null ) {
-            throw new SCRDescriptorFailureException("Descriptor file name must not be empty.");
-        }
 
         // finally the descriptors have to be written ....
         descriptorDir.mkdirs(); // ensure parent dir
 
         final List<String> fileNames = new ArrayList<String>();
-        final List<ComponentContainerContainer> containers = ComponentContainerUtil.split(components, options.isGenerateSeparateDescriptors());
+        final List<ComponentContainerContainer> containers = ComponentContainerUtil.split(components);
         for(final ComponentContainerContainer ccc : containers) {
             final SpecVersion globalVersion = module.getOptions().getSpecVersion();
 
-            final File useFile;
-            if ( ccc.className == null ) {
-                useFile = descriptorFile;
-            } else {
-                SpecVersion sv = null;
-                for(final ComponentContainer cc : ccc.components ) {
-                    if ( sv == null || sv.ordinal() < cc.getComponentDescription().getSpecVersion().ordinal() ) {
-                        sv = cc.getComponentDescription().getSpecVersion();
-                    }
+            SpecVersion sv = null;
+            for(final ComponentContainer cc : ccc.components ) {
+                if ( sv == null || sv.ordinal() < cc.getComponentDescription().getSpecVersion().ordinal() ) {
+                    sv = cc.getComponentDescription().getSpecVersion();
                 }
-                module.getOptions().setSpecVersion(sv);
-                useFile = new File(descriptorDir, ccc.className + ".xml");
             }
+            module.getOptions().setSpecVersion(sv);
+            final File useFile = new File(descriptorDir, ccc.className + ".xml");
             try {
                 ComponentDescriptorIO.generateXML(module, ccc.components, useFile, logger);
             } catch (final IOException e) {
-                throw new SCRDescriptorException("Unable to generate xml", descriptorFile.toString(), e);
+                throw new SCRDescriptorException("Unable to generate xml", useFile.toString(), e);
             } catch (final TransformerException e) {
-                throw new SCRDescriptorException("Unable to generate xml", descriptorFile.toString(), e);
+                throw new SCRDescriptorException("Unable to generate xml", useFile.toString(), e);
             } catch (final SAXException e) {
-                throw new SCRDescriptorException("Unable to generate xml", descriptorFile.toString(), e);
+                throw new SCRDescriptorException("Unable to generate xml", useFile.toString(), e);
             }
             fileNames.add(PARENT_NAME + '/' + useFile.getName());
 
